/*
 * Board functions for Atmark Techno Armadillo-640
 *
 * Copyright (C) 2017 Yasushi SHOJI <yashi@atmark-techno.com>
 *
 * SPDX-License-Identifier:	GPL-2.0+
 *
 */

#include <stdbool.h>
#include <linux/types.h>
#include <asm/types.h>
#include <asm/u-boot.h>
#include <asm/global_data.h>
#include <asm/arch/sys_proto.h>
#include <asm/arch/iomux.h>
#include <asm/arch/mx6-pins.h>
#include <asm/gpio.h>
#include <fuse.h>
#include <mmc.h>
#include <memalign.h>
#include "fec_mxc.h"
#include "board_usb.h"
#include "sdhc_mxc.h"
#include "board_wlan_rtc.h"
#include "board_pmic.h"
#include "board_kotai.h"
#ifdef CONFIG_A6_SUBBOARD
#include "a6_subboard.h"
#endif

DECLARE_GLOBAL_DATA_PTR;

#define RED_LED IMX_GPIO_NR(1, 5)
#define GREEN_LED IMX_GPIO_NR(1, 8)
#define YELLOW_LED IMX_GPIO_NR(1, 18)

#define BOOT_CFG1 IMX_GPIO_NR(3, 10)
#define BOOT_CFG2 IMX_GPIO_NR(3, 16)

#define MAINTENANCE_MODE_GPIO_CON9_1 IMX_GPIO_NR(1, 22)
#define MAINTENANCE_MODE_GPIO_SW1 IMX_GPIO_NR(1, 10)
static iomux_v3_cfg_t maintenance_mode_gpio_pads[] = {
	MX6_PAD_UART2_CTS_B__GPIO1_IO22 | MUX_PAD_CTRL(PAD_CTL_PUS_22K_UP | PAD_CTL_PUE),
	MX6_PAD_JTAG_MOD__GPIO1_IO10 | MUX_PAD_CTRL(PAD_CTL_HYS | PAD_CTL_PUS_100K_DOWN | PAD_CTL_DSE_DISABLE),
};

static iomux_v3_cfg_t status_led_gpio_pads[] = {
	MX6_PAD_GPIO1_IO05__GPIO1_IO05 | MUX_PAD_CTRL(PAD_CTL_DSE_240ohm),  /* LED3:Red */
	MX6_PAD_GPIO1_IO08__GPIO1_IO08 | MUX_PAD_CTRL(PAD_CTL_DSE_240ohm),  /* LED4:Green */
	MX6_PAD_UART1_CTS_B__GPIO1_IO18 | MUX_PAD_CTRL(PAD_CTL_DSE_240ohm), /* LED5:Yellow */
};

static void setup_maintenance_sw(void)
{
	imx_iomux_v3_setup_multiple_pads(maintenance_mode_gpio_pads,
					 ARRAY_SIZE(maintenance_mode_gpio_pads));
	gpio_direction_input(MAINTENANCE_MODE_GPIO_CON9_1);
	gpio_direction_input(MAINTENANCE_MODE_GPIO_SW1);
}

static void set_fdt_file(void)
{
	uint32_t val;
	int ret;

	if (env_get("fdt_file"))
		return;

	ret = fuse_read(EFUSE_KOTAI_BANK, EFUSE_KOTAI_WORD, &val);
	if (ret) {
		printf("Error: Can not read kotai-code\n");
		return;
	}
	switch (val) {
		case EFUSE_KOTAI_A640:
		case EFUSE_KOTAI_A640ES:
			env_set("fdt_file", "a640.dtb");
			break;
		case EFUSE_KOTAI_A610:
		case EFUSE_KOTAI_A610ES:
			env_set("fdt_file", "a610.dtb");
			break;
		default:
			printf("Error: unknown board. "
			       "set env `fdt_file' for boot.\n");
			break;
	}
}

static void set_mmcdev(void)
{
	if (!gpio_get_value(BOOT_CFG1) && gpio_get_value(BOOT_CFG2))
		env_set("mmcdev", "1");
	else
		env_set("mmcdev", "0");

}

static bool is_mtfc8gakajcn_1m(struct blk_desc *mmc_desc) {
	if (strcmp(mmc_desc->product + MTFC8GAKAJCN_1M_DEVICE_ID_OFFSET,
					MTFC8GAKAJCN_1M_DEVICE_ID) == 0)
		return true;
	return false;
}

static void mtfc8gakajcn_1m_workaround(void)
{
	struct mmc *mmc;
	struct blk_desc *mmc_desc;

	int offset;
	uint32_t blk_start;
	char *buf;

	mmc = find_mmc_device(MTFC8GAKAJCN_1M_DEVICE_NUM);
	if (!mmc) {
		printf("MTFC8GAKAJCN_1M workaround find_mmc_device error!\n");
		return;
	}

	if (mmc_init(mmc)) {
		printf("MTFC8GAKAJCN_1M workaround mmc_init error!\n");
		return;
	}

	mmc_desc = mmc_get_blk_desc(mmc);

	if (!is_mtfc8gakajcn_1m(mmc_desc))
		return;

	offset = CONFIG_ENV_OFFSET - mmc_desc->blksz;
	if (offset <= 0 || (offset % mmc_desc->blksz) != 0) {
		printf("MTFC8GAKAJCN_1M workaround offset value error!\n");
		return;
	}

	buf = memalign(ARCH_DMA_MINALIGN, mmc_desc->blksz);
	if (!buf) {
		printf("MTFC8GAKAJCN_1M workaround buffer allocate error!\n");
		return;
	}

	blk_start = ALIGN(offset, mmc->write_bl_len) / mmc->write_bl_len;
	if (blk_dread(mmc_desc, blk_start, 1, buf) != 1)
		printf("MTFC8GAKAJCN_1M workaround read error!\n");
	else if (blk_dwrite(mmc_desc, blk_start, 1, buf) != 1)
		printf("MTFC8GAKAJCN_1M workaround write error!\n");

	free(buf);
}

void coloured_LED_init(void)
{
	imx_iomux_v3_setup_multiple_pads(status_led_gpio_pads,
			       ARRAY_SIZE(status_led_gpio_pads));
	gpio_direction_output(GREEN_LED, 1);
}

int board_init(void)
{
	setup_maintenance_sw();
	setup_fec();
	setup_usb();

	return 0;
}

int board_late_init(void)
{
	wlan_rtc_board_late_init();
	pmic_late_init();
	set_fdt_file();
	set_mmcdev();
#ifdef CONFIG_A6_SUBBOARD
	a6_subboard_late_init();
#endif

	mtfc8gakajcn_1m_workaround();

	return 0;
}

int dram_init(void)
{
	gd->ram_size = imx_ddr_size();
	return 0;
}

#ifdef CONFIG_BOOTSTAGE
extern unsigned long __weak timer_read_counter(void);
ulong timer_get_boot_us(void)
{
	return timer_read_counter();
}
#endif

#if defined(CONFIG_SUPPORT_MAINTENANCE_MODE)
int is_maintenance_mode(void)
{
	return !gpio_get_value(MAINTENANCE_MODE_GPIO_CON9_1) || gpio_get_value(MAINTENANCE_MODE_GPIO_SW1);
}
#endif
